//
//  DCMotorController.m
//  DCMotor
//
//  Created by Phidgets on 2015-07-28.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "DCMotorController.h"

@implementation DCMotorController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetDCMotor_delete(&ch);
        ch = NULL;
    }
}


-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage view
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetDCMotor_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDCMotor_setOnVelocityUpdateHandler(ch, gotVelocityData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDCMotor_setOnBackEMFChangeHandler(ch, gotBackEMFData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDCMotor_setOnBrakingStrengthChangeHandler(ch, gotBrakingStrengthData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks

static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

static void gotVelocityData(PhidgetDCMotorHandle phid, void *context, double velocity){
    [(__bridge id)context performSelectorOnMainThread:@selector(onVelocityUpdateHandler:)
                                           withObject:[NSNumber numberWithDouble:velocity]
                                        waitUntilDone:NO];
}

static void gotBackEMFData(PhidgetDCMotorHandle phid, void *context, double backEMF){
    [(__bridge id)context performSelectorOnMainThread:@selector(onBackEMFChangeHandler:)
                                           withObject:[NSNumber numberWithDouble:backEMF]
                                        waitUntilDone:NO];
}

static void gotBrakingStrengthData(PhidgetDCMotorHandle phid, void *context, double brakingStrength){
    [(__bridge id)context performSelectorOnMainThread:@selector(onBrakingStrengthChangeHandler:)
                                           withObject:[NSNumber numberWithDouble:brakingStrength]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, and error events
- (void)onAttachHandler{
    double minAcceleration, maxAcceleration, acceleration;
    double velocity, minCurrentLimit, maxCurrentLimit, currentLimit, minCurrentRegulatorGain, maxCurrentRegulatorGain, currentRegulatorGain;
    Phidget_DeviceID deviceID;
    int backEMFSensingState;
    
    //Get information from channel which will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetDCMotor_getMinAcceleration(ch, &minAcceleration);
    PhidgetDCMotor_getMaxAcceleration(ch, &maxAcceleration);
    PhidgetDCMotor_getAcceleration(ch, &acceleration);
    PhidgetDCMotor_getVelocity(ch, &velocity);
    PhidgetDCMotor_getBackEMFSensingState(ch, &backEMFSensingState);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    
    
    //Configure the accleration slider with information read from channel
    [accelerationSlider setMinValue:minAcceleration];
    [accelerationSlider setMaxValue:maxAcceleration];
    [accelerationSlider setDoubleValue:acceleration];
    [accelerationMinLabel setStringValue:[NSString stringWithFormat:@"%0.2f", minAcceleration]];
    [accelerationMaxLabel setStringValue:[NSString stringWithFormat:@"%0.2f",maxAcceleration]];
    [accelerationText setStringValue:[NSString stringWithFormat:@"%0.2f",acceleration]];
    
    [velocityText setStringValue:[NSString stringWithFormat:@"%0.2f",velocity]];
    [velocitySlider setDoubleValue:velocity];
    
    //configure view
    switch(deviceID){
        case PHIDID_1060:
        case PHIDID_1064:
            [backEMFCheck setHidden:YES];
            [brakingStrengthBox setHidden:YES];
            [velocityChangeBox setHidden:NO];
            [backEMFBox setHidden:YES];
            break;
        case PHIDID_1065:
            [brakingStrengthText setStringValue:[NSString stringWithFormat:@"%0.2f",0.0]];
            [backEMFCheck setHidden:NO];
            [brakingStrengthBox setHidden:NO];
            [backEMFCheck setIntValue:backEMFSensingState];
            [velocityChangeBox setHidden:NO];
            [backEMFBox setHidden:NO];
            break;
        case PHIDID_DCC1001:
        case PHIDID_DCC1000:
            PhidgetDCMotor_getMinCurrentLimit(ch, &minCurrentLimit);
            PhidgetDCMotor_getMaxCurrentLimit(ch, &maxCurrentLimit);
            PhidgetDCMotor_getCurrentLimit(ch, &currentLimit);
            PhidgetDCMotor_getMinCurrentRegulatorGain(ch, &minCurrentRegulatorGain);
            PhidgetDCMotor_getMaxCurrentRegulatorGain(ch, &maxCurrentRegulatorGain);
            PhidgetDCMotor_getCurrentRegulatorGain(ch, &currentRegulatorGain);
            
            [extraSettingsBox setHidden:NO];
            [brakingStrengthBox setHidden:NO];
            [brakingStrengthChangeBox setHidden:NO];
            [velocityChangeBox setHidden:NO];
            [brakingStrengthText setStringValue:[NSString stringWithFormat:@"%0.2f",0.0]];
            [currentLimitLabel setStringValue:[NSString stringWithFormat:@"%0.2fA",currentLimit]];
            [currentLimitMinLabel setStringValue:[NSString stringWithFormat:@"%0.2f",minCurrentLimit]];
            [currentLimitMaxLabel setStringValue:[NSString stringWithFormat:@"%0.2f",maxCurrentLimit]];
            [currentLimitSlider setMinValue:minCurrentLimit];
            [currentLimitSlider setMaxValue:maxCurrentLimit];
            [currentLimitSlider setDoubleValue:currentLimit];
            if(deviceID == PHIDID_DCC1001)
                break;
            [currentRegulatorGainLabel setStringValue:[NSString stringWithFormat:@"%0.2f", currentRegulatorGain]];
            [currentRegulatorGainMinLabel setStringValue:[NSString stringWithFormat:@"%0.2f",minCurrentRegulatorGain]];
            [currentRegulatorGainMaxLabel setStringValue:[NSString stringWithFormat:@"%0.2f",maxCurrentRegulatorGain]];
            [currentRegulatorGainSlider setMinValue:minCurrentRegulatorGain];
            [currentRegulatorGainSlider setMaxValue:maxCurrentRegulatorGain];
            [currentRegulatorGainSlider setDoubleValue:currentRegulatorGain];
            [fanModeCombo selectItemAtIndex:2];
            break;
        default:
            break;
    }
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    if(![brakingStrengthBox isHidden])
        frame.size.height +=  brakingStrengthBox.frame.size.height;
    if(![extraSettingsBox isHidden])
        frame.size.height += extraSettingsBox.frame.size.height;
    if(deviceID == PHIDID_DCC1001){
        frame.size.height -= 156;
    }
        
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Reset
    [settingsBox setHidden:YES];
    [brakingStrengthBox setHidden:YES];
    [brakingStrengthChangeBox setHidden:YES];
    [backEMFBox setHidden:YES];
    [extraSettingsBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onVelocityUpdateHandler:(NSNumber *)velocity{
    [velocityChangeText setStringValue:[NSString stringWithFormat:@"%0.2f", velocity.doubleValue]];
}

- (void)onBackEMFChangeHandler:(NSNumber *)backEMF{
    [backEMFText setStringValue:[NSString stringWithFormat:@"%0.2f", backEMF.doubleValue]];
}

- (void)onBrakingStrengthChangeHandler:(NSNumber *)brakingStrength{
    [brakingStrengthChangeLabel setStringValue:[NSString stringWithFormat:@"%0.2f", brakingStrength.doubleValue]];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

#pragma mark GUI controls
- (IBAction)setVelocity:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setTargetVelocity(ch, [velocitySlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [velocityText setStringValue:[NSString stringWithFormat:@"%0.2f",[velocitySlider doubleValue]]];
}

- (IBAction)setAcceleration:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setAcceleration(ch, [accelerationSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [accelerationText setStringValue:[NSString stringWithFormat:@"%0.2f",[accelerationSlider doubleValue]]];
}

- (IBAction)setBrakingStrength:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setTargetBrakingStrength(ch, [brakingStrengthSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [brakingStrengthText setStringValue:[NSString stringWithFormat:@"%0.2f",[brakingStrengthSlider doubleValue]]];
}

- (IBAction)setCurrentLimit:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setCurrentLimit(ch, [currentLimitSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [currentLimitLabel setStringValue:[NSString stringWithFormat:@"%0.2fA",[currentLimitSlider doubleValue]]];
}
- (IBAction)setCurrentRegulatorGain:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setCurrentRegulatorGain(ch, [currentRegulatorGainSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [currentRegulatorGainLabel setStringValue:[NSString stringWithFormat:@"%0.2f",[currentRegulatorGainSlider doubleValue]]];
}
- (IBAction)setFanMode:(id)sender {
    PhidgetReturnCode result;
    Phidget_FanMode fanMode;
    if(fanModeCombo.indexOfSelectedItem == 0){
        fanMode = FAN_MODE_ON;
    }
    else if(fanModeCombo.indexOfSelectedItem == 1){
        fanMode = FAN_MODE_OFF;
    }
    else{
        fanMode = FAN_MODE_AUTO;
    }
    result = PhidgetDCMotor_setFanMode(ch, fanMode);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)setEMFSensing:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setBackEMFSensingState(ch, [backEMFCheck intValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
}

- (IBAction)zeroVelocity:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetDCMotor_setTargetVelocity(ch, 0);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [velocityText setStringValue:[NSString stringWithFormat:@"%0.2f",0.0]];
    [velocitySlider setDoubleValue:0.0];
}

@end
